import settings

import re
import requests

from django.http import HttpResponse

import logging
logger = logging.getLogger(__name__)


class LocalizeSEOMiddleware(object):

    def __init__(self, *args, **kwargs):
        regex_str = "|".join(settings.USER_AGENTS)
        regex_str = ".*?(%s)" % regex_str
        self.USER_AGENT_REGEX = re.compile(regex_str, re.IGNORECASE)
        self.BASE_URL = settings.PRERENDER_URL
        self.session = requests.Session()

    def process_request(self, request):
        # import ipdb; ipdb.set_trace()
        if not settings.ENABLED:
            return

        # TBI makesure rootDomain is available
        if self.ignore_request(request):
            return

        if "HTTP_USER_AGENT" not in request.META:
            return

        if not self.USER_AGENT_REGEX.match(request.META["HTTP_USER_AGENT"]):
            return

        render_url = self.build_render_url(request)  # TBI

        try:
            return self.get_response(render_url)  # TBI
        except Exception as e:
            logger.Exception(e)

    def ignore_request(self, request):
        for url in settings.IGNORE_URLS:
            if url in request.path:
                return True

        extension = None
        last_dot = request.path.rfind(".")
        if last_dot == -1:
            return False

        extension = request.path[last_dot:]
        return extension and extension in settings.IGNORE_EXTENSIONS

    def build_render_url(self, request):
        url = '{scheme}://{host}{path}'.format(
            scheme=request.scheme,
            host=request.get_host(),
            path=request.path,
        )

        if "//" not in url:
            raise ValueError("Invalid url: %s" % url)
        render_url = "%s?url=%s" % (self.BASE_URL, url)
        return render_url

    def get_response(self, url):
        response = self.session.get(url)
        assert response.status_code < 500
        r = HttpResponse(response)
        r.status_code = response.status_code
        return r



from distutils.core import setup
# import ipdb; ipdb.set_trace()
setup(
  name = 'django_localizejs_seo',
  packages = ['django_localizejs_seo'], 
  version = '0.1',
  description = 'A Python Django prerender middleware for Localize JS',
  author = 'Yoland Yan',
  author_email = 'yoland68@gmail.com',
  url = 'https://github.com/yoland68/django_localizejs_seo', 
  download_url = 'https://github.com/yoland68/django_localizejs_seo/tarball/0.1', 
  keywords = ['django', 'seo', 'prerender'], 
  classifiers = [],
  install_requires=[
        "requests",
  ],
)


from django.conf import settings as django_settings

ENABLED = getattr(django_settings, 'LOCALIZE_SEO_ENABLED', not django_settings.DEBUG)

IGNORE_URLS = frozenset(getattr(django_settings, 'LOCALIZE_SEO_IGNORE_URLS', ['/sitemap.xml']))

IGNORE_EXTENSIONS = frozenset(getattr(django_settings, 'LOCALIZE_SEO_IGNORE_EXTENSIONS', (
    ".js",
    ".css",
    ".xml",
    ".less",
    ".png",
    ".jpg",
    ".jpeg",
    ".gif",
    ".pdf",
    ".doc",
    ".txt",
    ".ico",
    ".rss",
    ".zip",
    ".mp3",
    ".rar",
    ".exe",
    ".wmv",
    ".doc",
    ".avi",
    ".ppt",
    ".mpg",
    ".mpeg",
    ".tif",
    ".wav",
    ".mov",
    ".psd",
    ".ai",
    ".xls",
    ".mp4",
    ".m4a",
    ".swf",
    ".dat",
    ".dmg",
    ".iso",
    ".flv",
    ".m4v",
    ".torrent",
)))

USER_AGENTS = frozenset(getattr(django_settings, 'LOCALIZE_SEO_USER_AGENTS', (
    "bot",
    "crawler",
    "baiduspider",
    "80legs",
    "mediapartners-google",
    "adsbot-google"
)))

PRERENDER_URL = getattr(django_settings, 'LOCALIZE_SEO_PRERENDER_URL', "https://prerender.localizejs.com/api/prerender/get")